// Georgui Rekouts, Nov 1994
// nodeview.cpp - Base class for the viewer. Handles the diplaying of a node

#include "treelist.h"

//------------------------------------------ Node View constructor
TNodeView::TNodeView()
{
  BmEmpty = NULL;
  BmOpen  = NULL;
  BmClose = NULL;
  BmNode  = NULL;
}

//------------------------------------------ Node View constructor
TNodeView::~TNodeView()
{
  delete BmOpen;
  delete BmClose;
  delete BmNode;
}

//---------------------------------------------------- Bitmap management
void TNodeView::DrawBitmap(TDC &dc, TBitmap *bitmap, TPoint point)
{
  TMemoryDC dcMem(dc);
  dcMem.SelectObject(*bitmap);
  dc.BitBlt(point.x, point.y, bitmap->Width(), bitmap->Height(),
	 dcMem, 0, 0, SRCCOPY);
  dcMem.RestoreBitmap();
}

//-------------------------------------------------- Get bitmap
TBitmap *TNodeView::GetBitmap(TNode *node)
{
  TBitmap *bitmap;

  if (node->IsParent())
  {
	 if (node->IsOpen())
		bitmap = GetOpenBitmap();
	 else
		bitmap = GetCloseBitmap();
  }
  else
	 bitmap = GetNodeBitmap();

  return bitmap;
}

//------------- Paint, derived classes should override these
void TNodeView::Paint( DRAWITEMSTRUCT &drawInfo, TNode *node,
  TFont *font, int indentUnits)
{
  //............ extracting draw info
  TDC   dc(drawInfo.hDC);
  TRect rect(drawInfo.rcItem);

  TBitmap *bitmap = GetBitmap(node);

  if( node->GetIndentLevel() > 0 )
  {
	  TNode* parent = node;
	  // looping through the nested levels
	  for( int i = node->GetIndentLevel() - 1; i >= 0; i-- )
	  {
		  // determining if node is first on the list
		  TNodeListIterator iter( *((parent->GetParent())->GetChildList()));
		  int counter = 0;
		  while (iter)
		  {
			  TNode *current = iter.Current();
			  iter++;
			  counter++;
			  if( current == parent )
				  break;
		  }

		  TPoint point( rect.left + indentUnits * i + bitmap->Width() / 2,
				rect.top );

		  // if node first on the list that means it last branch on the tree
		  if( counter != 1 )
		  {
			  dc.MoveTo( point );
			  point.y = rect.bottom;
			  dc.LineTo( point );

			  // if most inner level
			  if( i == node->GetIndentLevel() - 1 )
			  {
				  point.y = ( rect.bottom + rect.top ) / 2;
				  dc.MoveTo( point );
				  point.x += indentUnits;
				  dc.LineTo( point );
			  }
		  }
		  else if( i == node->GetIndentLevel() - 1 )
		  {
			  dc.MoveTo( point );
			  point.y = ( rect.bottom + rect.top ) / 2;
			  dc.LineTo( point );
			  point.x += indentUnits;
			  dc.LineTo( point );
		  }

		  parent = parent->GetParent();
	  }
  }
  //....................... draw bitmap
  rect.left += indentUnits * node->GetIndentLevel();

  if (bitmap != NULL)
  {
	 TSize size(-1, -1);
	 GetSize(node, font, size);
	 TPoint point;
	 point.x = rect.left;
	 point.y = rect.top + (size.cy / 2 - bitmap->Height() / 2);
	 DrawBitmap(dc, bitmap, point);
	 rect.left += bitmap->Width() + 1;
  }
  //....................... Pass new rect back
  drawInfo.rcItem = rect;
}

//------------------------------------------------ Get Size
void TNodeView::GetSize(TNode *node, TFont *font, TSize &size)
{
  TBitmap *bitmap = GetBitmap(node);
  //.............. returns max of font and bitmap heights
  if (bitmap != NULL)
  {
	 size.cx = bitmap->Width();
	 size.cy = bitmap->Height() + 2;
  }
  if (font)
  {
	 size.cy = (size.cy > GetFontHeight(*font)) ? size.cy : GetFontHeight(*font);
  }
}

//------------------------------------------------ Set XXXX Bitmap
void TNodeView::SetEmptyBitmap(TBitmap *bmEmpty)
{
  if (BmEmpty)
	 delete BmEmpty;
  BmEmpty = bmEmpty;
}

void TNodeView::SetOpenBitmap(TBitmap *bmOpen)
{
  if (BmOpen)
	 delete BmOpen;
  BmOpen = bmOpen;
}

void TNodeView::SetCloseBitmap(TBitmap *bmClose)
{
  if (BmClose)
	 delete BmClose;
  BmClose = bmClose;
}

void TNodeView::SetNodeBitmap(TBitmap *bmNode)
{
  if (BmNode)
	 delete BmNode;
  BmNode = bmNode;
}

//------------------------------------------------ Font management
int TNodeView::GetFontHeight(HFONT hFont)
{
  int height = 0;
  if( hFont != NULL )
  {
	  HDC hdc = CreateIC("DISPLAY", NULL, NULL, NULL);
	  TDC dc(hdc);
	  TFont font(hFont);
	  dc.SelectObject(font);
	  TEXTMETRIC tm;
	  dc.GetTextMetrics(tm);
	  dc.RestoreFont();
	  DeleteDC(hdc);
	  height = tm.tmHeight;
  }
  return height;
}

//------------------------------------------ needed operators for TListImp
TNodeView::TNodeView(const TNodeView &copyNode)
{
  *this = copyNode;
}

TNodeView& TNodeView::operator = (const TNodeView &copyNode)
{
  SetEmptyBitmap(copyNode.GetEmptyBitmap());
  SetOpenBitmap(copyNode.GetOpenBitmap());
  SetCloseBitmap(copyNode.GetCloseBitmap());
  SetNodeBitmap(copyNode.GetNodeBitmap());
  return *this;
}

int TNodeView::operator == (const TNodeView &) const
{
  return FALSE;
}


